{% extends "base.html" %}

{% block title %}
	&raquo; DNOs &raquo; {{llfc.dno.dno_code}} &raquo; 
	LLFCs &raquo; {{llfc.code}} {{llfc.valid_from|hh_format}} &raquo; Edit
{% endblock %}

{% block nav %}
	<a href="/e/dnos">DNOs</a> &raquo; 
	<a href="/e/dnos/{{llfc.dno.id}}">{{llfc.dno.dno_code}}</a> &raquo; 
	<a href="/e/llfcs?dno_id={{llfc.dno.id}}">LLFCs</a> &raquo;
	<a
			href="/e/llfcs/{{llfc.id}}"
			>{{llfc.code}} {{llfc.valid_from|hh_format}}</a> &raquo;
	Edit
{% endblock %}

{% block content %}
	<form action="/e/llfcs/{{llfc.id}}/edit" method="post">
		<fieldset>
			<legend>Update LLFC</legend>	
			<label>Voltage Level</label>
			<select name="voltage_level_id">
				{% for voltage_level in voltage_levels %}
					{{input_option(
						'voltage_level_id', voltage_level.id,
						voltage_level.code, llfc.voltage_level.id)}}
				{% endfor %}
			</select>
			<label>Substation?</label>
			{{input_checkbox('is_substation', llfc.is_substation)}}
			<input type="submit" value="Update">
		</fieldset>
	</form>

{% endblock %}
