{% extends "base.html" %}

{% block title %}
	CSV Sites Displaced TRIAD
{% endblock %}

{% block nav %}
	CSV Sites Displaced TRIAD
{% endblock %}

{% block content %}

	<form action="/reports/181">
		<fieldset>
			<legend>Download Sites Displaced TRIAD CSV</legend>	
			<label>For financial year ending March</label> {{input_text('year', year, 4, 4)}}
			<input type="submit" value="Download">
		</fieldset>
	</form>

{% endblock %}
